import os.path, funcy
import biobox.image.volume as vol

from functools import partial

def remap_entries(xs):

    def remap(path_dict, x):
        f = lambda i: os.path.join(path_dict[vol.host_directory(i)], os.path.basename(i))
        return funcy.update_in(x, ['value'], f)

    paths = funcy.pluck('value', xs)
    return map(partial(remap, vol.create_host_container_directory_mapping(paths)), xs)

def remap_biobox_input_paths(args):
    return map(partial(funcy.walk_values, remap_entries), args)

def generate_biobox_file_content(args):
    import yaml
    output = {"version" : "0.9.0", "arguments" : args}
    return yaml.safe_dump(output, default_flow_style = False)

def create_biobox_directory(content):
    import tempfile
    dir_ = tempfile.mkdtemp()
    with open(os.path.join(dir_, "biobox.yaml"), "w") as f:
        f.write(content)
    return dir_



import os.path
import biobox.util

def clean_up_container(id_):
    if not "CIRCLECI" in os.environ:
        biobox.util.client().remove_container(id_)


def short_read_fastq():
    return os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            '..', 'tmp/data/reads.fq.gz')

def biobox_args():
    return [{"fastq" : [
        {"id" : 0 , "value" : short_read_fastq(), "type": "paired"}]}]



import tempfile, os.path
import biobox.config as cnf

dir_1 = tempfile.mkdtemp()
dir_2 = tempfile.mkdtemp()

reads_1 = os.path.join(dir_1, 'reads_1')
reads_2 = os.path.join(dir_1, 'reads_2')
reads_3 = os.path.join(dir_2, 'reads_3')

def create_args(x):
    return [{"fastq" : x}]

def test_remap_args_with_single_file():
    input    = [{"id" : 0 , "value" : reads_1, "type": "paired"}]
    expected = [{"id" : 0 , "value" : "/bbx/mount/0/reads_1", "type": "paired"}]
    assert cnf.remap_biobox_input_paths(create_args(input)) == create_args(expected)

def test_remap_args_with_two_files_in_same_dir():
    input    = [{"id" : 0 , "value" : reads_1, "type": "paired"},
                {"id" : 1 , "value" : reads_2, "type": "paired"}]
    expected = [{"id" : 0 , "value" : "/bbx/mount/0/reads_1", "type": "paired"},
                {"id" : 1 , "value" : "/bbx/mount/0/reads_2", "type": "paired"}]
    assert cnf.remap_biobox_input_paths(create_args(input)) == create_args(expected)

def test_remap_args_with_two_files_in_different_dir():
    input    = [{"id" : 0 , "value" : reads_1, "type": "paired"},
                {"id" : 1 , "value" : reads_3, "type": "paired"}]
    output = cnf.remap_biobox_input_paths(create_args(input))
    assert not os.path.dirname(output[0]['fastq'][0]['value']) == \
            os.path.dirname(output[0]['fastq'][1]['value'])



import docker.utils, funcy
import biobox.util         as util
import biobox.config       as cfg
import biobox.image.volume as vol

from functools import partial

def prepare_biobox_file(config):
    """
    Creates a biobox file in a temporary directory and returns a
    Docker volume string for the location.
    """
    f = funcy.compose(
            vol.biobox_file,
            cfg.create_biobox_directory,
            cfg.generate_biobox_file_content,
            cfg.remap_biobox_input_paths)
    return f(config)

def prepare_input_volumes(config):
    """
    Creates volume strings for all input arguments defined in
    the biobox config.
    """
    f = funcy.compose(
            vol.create_input_volume_strings,
            partial(funcy.pluck, 'value'),
            funcy.flatten,
            funcy.cat,
            partial(map, funcy.itervalues))
    return f(config)

def prepare_volumes(config, output_directory):
    return prepare_input_volumes(config)  + \
            [prepare_biobox_file(config)] + \
            [vol.output(output_directory)]

def create_container(image, config, output_directory, task = "default", docker_args = {}):
    volumes = prepare_volumes(config, output_directory)
    docker_args['volumes']     = map(vol.get_host_path, volumes)
    docker_args['host_config'] = util.client().create_host_config(binds=volumes)
    return util.client().create_container(image, task, **docker_args)



import os
from functools import partial

BIOBOX_INPUT_MOUNT_LOC = "/bbx/mount"

def create_volume_string(host_dir, container_dir, read_only = True):
    """
    Creates the volume strings used by the Docker daemon for mounting volumes
    """
    access = "ro" if read_only else "rw"
    return ":".join([os.path.abspath(host_dir), container_dir, access])


def output(host_directory_path):
    """
    Creates volume string for the biobox output directory path
    """
    return create_volume_string(host_directory_path, "/bbx/output", False)


def input(host_directory_path):
    """
    Creates volume string for the biobox input directory path
    """
    return create_volume_string(host_directory_path, BIOBOX_INPUT_MOUNT_LOC, True)


def biobox_file(host_directory_path):
    """
    Creates volume string for the directory containing biobox.yaml
    """
    return create_volume_string(host_directory_path, "/bbx/input")


def get_host_path(volume_string):
    """
    Returns the host path from a Docker volume string
    """
    return volume_string.split(":")[0]


def host_directory(x):
    return os.path.dirname(os.path.abspath(x))


def create_host_container_directory_mapping(paths):
    """
    Given a list of dirs on the host returns a dictionary mapping
    them to dirs within the Docker container
    """

    def container_directory(index):
        return os.path.join(BIOBOX_INPUT_MOUNT_LOC, str(index))

    uniq_paths = set(map(host_directory, paths))
    return dict(map(lambda (i, v): (v, container_directory(i)), enumerate(uniq_paths)))


def create_input_volume_strings(paths):
    """
    Given a list of dirs on the host returns the volume strings used by the
    Docker daemon to mount the dirs into the container
    """
    return map(partial(apply, create_volume_string),
            create_host_container_directory_mapping(paths).items())
